//
// HH:   Traub & Miles 1991 "Neuronal Networks of the Hippocampus", Cambridge Univ Press.
// ITCa: Destexhe et al (1996) J Neurosci16:169-1856.
//
//    y[0]  v : voltage
//    y[1]  m : activation parameter for Na channel
//    y[2]  h : inactivation parameter for Na channel
//    y[3]  n : activation parameter for K channel
//    y[4]  m : activation parameter for TCa channel
//    y[5]  h : inactivation parameter for TCa channel
//  



#include <math.h>
#include "solveHH.h"
CsolveHH::CsolveHH(){}
CsolveHH::~CsolveHH(){}

void CsolveHH::initValues()
{
	double tau, inf;
	double v;

	Cm  = 20.0;   // (pF)
	Dm = 60.0;  // (micrometer) diameter of the cell
	Lm = 70.0;  // (micrometer) length of the cell

	Iinj = -2000; //2000.0;  //(pA)
	Ton = 20;
	Toff = 220;// 40;

	Eleak = -65.0;   // (mV)
	ENa = 50.0;   // (mV)
	EK = -100.0;   // (mV)
	ECa = 50.0;   // (mV)

	dGleak = 0.002;// (mS/cm^2)
	dGNa   = 0.05; //0.53;   // (mS/cm^2)
	dGK    = 0.15; // 0.13;  // (mS/cm^2)
	dGTCa  = 0.03;  //  (mS/cm^2)

	vtraub  = -55.0; //   (mV)

	Am = 3.14 * Dm * Lm;    // (squre_micrometer) surface area
	Cm = 0.01 * Am;
	InvCm = 1/Cm;

	fac = 10;  // conversion factor (mS/cm^2) -> (pS/micrometer^2)

	Gleak = Am * dGleak * fac;
	GNa   = Am * dGNa * fac;
	GK    = Am * dGK * fac;
	GTCa  = Am * dGTCa * fac;

	y[0] = Eleak;
	get_Na_m(&tau, &inf);
	y[1] = inf;
	get_Na_h(&tau, &inf);
	y[2] = inf;
	get_K_n(&tau, &inf);
	y[3] = inf;
	get_TCa_m(&tau, &inf);
	y[4] = inf;
	get_TCa_h(&tau, &inf);
	y[5] = inf;
}


void CsolveHH::get_Na_m(double* tau, double* inf)
{
	double a, b;
	double v2 = y[0]-vtraub;

	a = 0.32 * (13-v2) / ( exp((13-v2)/4) - 1);
	b = 0.28 * (v2-40) / ( exp((v2-40)/5) - 1);
	*tau = 1 / (a + b);
	*inf = a / (a + b);
}
void CsolveHH::get_Na_h(double* tau, double* inf)
{
	double a, b;
	double v2 = y[0]-vtraub;
	a = 0.128 * exp((17-v2)/18);
	b = 4 / ( 1 + exp((40-v2)/5) );
	*tau = 1 / (a + b);
	*inf = a / (a + b);
}
void CsolveHH::get_K_n(double* tau, double* inf)
{
	double a, b;
	double v2 = y[0] - vtraub;
	a = 0.032 * (15-v2) / ( exp((15-v2)/5) - 1);
	b = 0.5 * exp((10-v2)/40);
    *tau = 1 / (a + b);
	*inf = a / (a + b);
}
void CsolveHH::get_TCa_m(double* tau, double* inf)
{
	double vshift=0;
	double vs = y[0] + vshift;

	*tau = ( 3 + 1.0 / ( exp((vs+25)/10) + exp(-(vs+100)/15) ) ); 
	*inf = 1.0 / ( 1 + exp(-(vs+50)/7.4) );
}
void CsolveHH::get_TCa_h(double* tau, double* inf)
{
	double vshift=0;
	double vs = y[0] + vshift;
	*tau = ( 85 + 1.0 / ( exp((vs+46)/4) + exp(-(vs+405)/50) ) ) ;
	*inf = 1.0 / ( 1 + exp((vs+78)/5.0) );
}

void CsolveHH::rhs(double x, double* y, double* dy){
	double INa, IK, ITCa;
	double tau, inf;
	double shift = 0;
	double v = y[0];
	double c = (x<Ton || x>Toff)? 0 : Iinj;
// voltage
	INa = GNa * pow(y[1],3.0) * y[2]  * (v-ENa);   // INa = GNa * m^3 * h * (v-Ena)
	IK = GK * pow(y[3],4.0) * (v-EK);             // IK = GK * n^4 (v-EK) 
	ITCa = GTCa * pow(y[4],2.0) * y[5] * (v - ECa);
	dy[0] = InvCm * (c - Gleak * (v-Eleak) - INa - IK - ITCa );
// Na channel
	get_Na_m(&tau, &inf);
    dy[1] = (inf - y[1]) / tau;
	get_Na_h(&tau, &inf);
    dy[2] = (inf - y[2]) / tau;
// K channel (delayed rectifier)
	get_K_n(&tau, &inf);
	dy[3] = (inf - y[3]) / tau;
// T-type Ca channel
	get_TCa_m(&tau, &inf);
    dy[4] = (inf - y[4]) / tau;
	get_TCa_h(&tau, &inf);
    dy[5] = (inf - y[5]) / tau;
}


